@@ -49,11 +49,9 @@ static void  nautilus_burn_register_type (GTypeModule *module);
 static GObjectClass *parent_class;
 
 static void
-write_activate_cb (NautilusMenuItem *item,
-		   gpointer user_data)
+launch_process (char **argv, GtkWindow *parent)
 {
 	GError *error;
-	char *argv[] = {BINDIR "/nautilus-cd-burner", NULL};
 
 	error = NULL;
 	if (!g_spawn_async (NULL,
@@ -66,18 +64,26 @@ write_activate_cb (NautilusMenuItem *item,
 		eel_show_error_dialog (_("Unable to launch the cd burner application"),
 				       error->message,
 				       _("Can't launch cd burner"),
-				       GTK_WINDOW (user_data));
+				       GTK_WINDOW (parent));
 		g_error_free (error);
 	}
 }
 
+static void
+write_activate_cb (NautilusMenuItem *item,
+		   gpointer user_data)
+{
+	char *argv[] = {BINDIR "/nautilus-cd-burner", NULL};
+	
+	launch_process (argv, GTK_WINDOW (user_data));
+}
+
 static void
 write_iso_activate_cb (NautilusMenuItem *item,
 		       gpointer user_data)
 {
 	NautilusFileInfo *file;
 	GtkWidget *window;
-	GError *error;
 	char *argv[3];
 	char *uri;
 	char *image_name;
@@ -99,20 +105,8 @@ write_iso_activate_cb (NautilusMenuItem *item,
 	argv[1] = image_name;
 	argv[2] = NULL;
 
-	error = NULL;
-	if (!g_spawn_async (NULL,
-			    argv, NULL,
-			    0,
-			    NULL, NULL,
-			    NULL,
-			    &error)) {
-		eel_show_error_dialog (_("Unable to launch the cd burner application"),
-				       error->message,
-				       _("Can't launch cd burner"),
-				       GTK_WINDOW (user_data));
-		g_error_free (error);
-	}
-
+	launch_process (argv, GTK_WINDOW (user_data));
+	
 	g_free (image_name);
 	
 }
