@@ -23,6 +23,7 @@
 */
 
 int make_iso (const char *filename, const char *label,
-		gboolean warn_low_space, gboolean use_joliet);
+		gboolean warn_low_space, gboolean use_joliet,
+		gboolean debug);
 void make_iso_cancel (void);
 
