@@ -131,7 +131,17 @@ request_op (gint32 operation,
 	return reply->result;
 }
 
+static GnomeVFSResult
+remove_file_helper (char *method, char *path)
+{
+	MappingReply reply;
+	GnomeVFSResult res;
 
+	res = request_op (MAPPING_REMOVE_FILE, method,
+		    path, NULL, FALSE, &reply);
+	destroy_reply (&reply);
+	return res;
+}
 
 static GnomeVFSResult
 do_open (GnomeVFSMethod *method,
@@ -419,29 +429,41 @@ do_read_directory (GnomeVFSMethod *method,
 	GnomeVFSResult res;
 	char *name;
 	char *backingfile;
+	char *path;
 	GnomeVFSURI *file_uri;
 
 	DEBUG_PRINT (("do_read_directory: %p\n", method_handle));
-	
-	if (handle->pos >= handle->n_items) {
-		return GNOME_VFS_ERROR_EOF;
-	}
 
-	name = handle->listing[handle->pos*2];
-	backingfile = handle->listing[handle->pos*2+1];
-	++handle->pos;
-	
-	if (backingfile == NULL) {
-		file_info->name = g_strdup (name);
-		fill_in_directory_info (file_info);
-	} else {
+	while (TRUE) {
+		if (handle->pos >= handle->n_items) {
+			return GNOME_VFS_ERROR_EOF;
+		}
+
+		name = handle->listing[handle->pos*2];
+		backingfile = handle->listing[handle->pos*2+1];
+		++handle->pos;
+
+		if (backingfile == NULL) {
+			file_info->name = g_strdup (name);
+			fill_in_directory_info (file_info);
+			break;
+		}
+
 		file_uri = get_uri (backingfile);
 		res = gnome_vfs_get_file_info_uri_cancellable
 			(file_uri, file_info, handle->options, context);
 		gnome_vfs_uri_unref (file_uri);
-		
+
+		if (res == GNOME_VFS_ERROR_NOT_FOUND) {
+			path = g_build_filename (handle->dirname, name, NULL);
+			remove_file_helper (handle->root, path);
+			g_free (path);
+			continue;
+		}
+
 		g_free (file_info->name);
 		file_info->name = g_strdup (name);
+		break;
 	}
 	
 	return GNOME_VFS_OK;
@@ -573,7 +595,6 @@ do_unlink (GnomeVFSMethod *method,
 {
 	GnomeVFSResult res;
 	char *path;
-	MappingReply reply;
 
 	DEBUG_PRINT (("do_remove_directory: %s\n", gnome_vfs_uri_to_string (uri, 0)));
 	
@@ -582,14 +603,8 @@ do_unlink (GnomeVFSMethod *method,
 		return GNOME_VFS_ERROR_INVALID_URI;
 	}
 
-	
-	res = request_op (MAPPING_REMOVE_FILE,
-			  uri->method_string,
-			  path,
-			  NULL, FALSE,
-			  &reply);
+	res = remove_file_helper (uri->method_string, path);
 	g_free (path);
-	destroy_reply (&reply);
 
 	return res;
 }
